/*
 *
 * apps/videoServer/VideoClient.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "VideoClient.H"

#include <nucleo/image/sink/serverpushImageSink.H>

#include <iostream>
#include <stdexcept>

// ---------------------------------------------------------------

VideoClient::VideoClient(ConfigDict &config, TcpConnection *c) {
  serverConfig = config ;
  connection = c ;

  std::string machine="?", user="?" ;
  try { machine = connection->machineLookUp() ; } catch (...) {}
  try { user = connection->userLookUp() ; } catch (...) {}
  std::string referer="?", agent="?" ;
  request.getHeader("referer", referer) ;
  request.getHeader("user-agent", agent) ;

  Notifier notifier() ;

  std::string service = "streamImages" ;

  if (service=="redirect")
    ; // ...
  else if (service=="sendFile")
    ; // ...
  else if (service=="sendImage")
    ; // ...
  else {
    subscribeTo(connection) ;
    std::string uri = serverConfig.get("image-source", std::string("videoin:")) ;
    source = ImageSource::create(uri) ;
    subscribeTo(source) ;
    source->start() ;   
    sink = new serverpushImageSink(connection->getFd()) ;
    subscribeTo(sink) ;
    sink->start() ;
  }
}

// ---------------------------------------------------------------

void
VideoClient::react(Observable *obs) {
  bool terminate = false ;

  if (obs==connection)
    terminate = true ;
  else if (obs==source) {
    try {
	 if (source->getNextImage(&img)) sink->handle(&img) ;
    } catch (std::runtime_error e) {
	 // std::cerr << e.what() << std::endl ;
	 terminate = true ;
    }
  }

  if (!source->isActive()) terminate = true ;
  if (!sink->isActive()) terminate = true ;

  if (terminate) delete this ;
}

// ---------------------------------------------------------------

VideoClient::~VideoClient(void) {
  std::cerr << "Deleting VideoClient " << this << std::endl ;

  unsubscribeFrom(source) ;
  delete source ;
  unsubscribeFrom(sink) ;
  delete sink ;
  unsubscribeFrom(connection) ;
  delete connection ;
}
